/* 
 * 	Copyright (c) 2010 Colorado State University
 * 
 *	Permission is hereby granted, free of charge, to any person
 *      obtaining a copy of this software and associated documentation
 *	files (the "Software"), to deal in the Software without
 *	restriction, including without limitation the rights to use,
 *	copy, modify, merge, publish, distribute, sublicense, and/or
 *	sell copies of the Software, and to permit persons to whom
 *	the Software is furnished to do so, subject to the following
 *	conditions:
 *
 *	The above copyright notice and this permission notice shall be
 *	included in all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *	OTHER DEALINGS IN THE SOFTWARE.
 * 
 * 
 *  File: configdefaults.h
 * 	Authors: Kevin Burnett
 *  Date: August 2, 2008
 */

#ifndef CONFIGDEFAULTS_H_
#define CONFIGDEFAULTS_H_

/* Sets the maximum number of characters that can appear in any XML
 * element within the BGPmon configuration file.  XML values with more 
 * characters than this are truncated.
 */
#define XML_MAX_CHARS 255

// XML Tags
#define XML_BGPMON_TAG "BGPmon"

// Login XML Tags
#define XML_LOGIN_TAG "LOGIN"
#define XML_LOGIN_PORT_TAG "LOGIN_PORT"
#define XML_LOGIN_ADDRESS_TAG "LOGIN_ADDRESS"
#define XML_LOGIN_ACCESS_PASSWORD_TAG "ACCESS_PASSWORD"
#define XML_LOGIN_ENABLE_PASSWORD_TAG "ENABLE_PASSWORD"
#define XML_LOGIN_CLI_ACL_TAG "CLI_ACL"
#define XML_LOGIN_CLIENT_UPDATE_ACL_TAG "CLIENT_UPDATE_ACL"
#define XML_LOGIN_CLIENT_RIB_ACL_TAG "CLIENT_RIB_ACL"
#define XML_LOGIN_MRT_ACL_TAG "MRT_ACL"

// AccessControlList XML Tags
#define XML_ACL_TAG "ACCESS_CONTROL_LIST"
#define XML_ACL_LIST "LIST"
#define XML_ACL_LIST_NAME "NAME"
#define XML_ACL_LIST_RULE "RULE"
#define XML_ACL_LIST_RULE_INDEX "INDEX"
#define XML_ACL_LIST_RULE_ADDRESS "ADDRESS"
#define XML_ACL_LIST_RULE_MASK "MASK"
#define XML_ACL_LIST_RULE_ALLOWED "ALLOWED"

// Queue Tags
#define XML_QUEUE_TAG "QUEUE"
#define XML_QUEUE_PACING_ON "QUEUE_PACING_ON"
#define XML_QUEUE_PACING_OFF "QUEUE_PACING_OFF"
#define XML_QUEUE_ALPHA "ALPHA"
#define XML_QUEUE_MIN_WRITES_LIMIT "QUEUE_MIN_WRITES"
#define XML_QUEUE_PACING_INTERVAL "QUEUE_PACING_INTERVAL"
#define XML_QUEUE_LOG_INTERVAL "QUEUE_LOG_INTERVAL"

// Clients Control Tags
#define XML_CLIENTS_CTR_TAG "CLIENTS"
#define XML_CLIENTS_CTR_RIB_LISTEN_ADDR "RIB_LISTEN_ADDR"
#define XML_CLIENTS_CTR_RIB_LISTEN_PORT "RIB_LISTEN_PORT"
#define XML_CLIENTS_CTR_RIB_MAX_CLIENTS "RIB_MAX_CLIENTS"
#define XML_CLIENTS_CTR_UPDATES_LISTEN_ADDR "UPDATES_LISTEN_ADDR"
#define XML_CLIENTS_CTR_UPDATES_LISTEN_PORT "UPDATES_LISTEN_PORT"
#define XML_CLIENTS_CTR_UPDATES_MAX_CLIENTS "UPDATES_MAX_CLIENTS"
#define XML_CLIENTS_CTR_ENABLED "ENABLED"
#define XML_CLIENTS_CTR_BGPMON_ID "BGPMON_ID"

// Mrts Control Tags
#define XML_MRTS_CTR_TAG "MRTS"
#define XML_MRTS_CTR_LISTEN_ADDR "LISTEN_ADDR"
#define XML_MRTS_CTR_LISTEN_PORT "LISTEN_PORT"
#define XML_MRTS_CTR_ENABLED "ENABLED"
#define XML_MRTS_CTR_MAX_MRTS "MAX_MRTS"
#define XML_MRTS_CTR_LABEL_ACTION "LABEL_ACTION"

// Chains Tags
#define XML_CHAINS_LIST_TAG "CHAINS"
#define XML_CHAIN_TAG "CHAIN"
#define XML_CHAIN_ADDR "CHAIN_ADDR"
#define XML_U_CHAIN_PORT "U_CHAIN_PORT"
#define XML_R_CHAIN_PORT "R_CHAIN_PORT"
#define XML_CHAIN_MD5PW "MD5_PASSWD"
#define XML_CHAIN_ENABLED "ENABLED"
#define XML_CHAIN_RETRY_INTERVAL "RETRY_INTERVAL"

// Peering and labeling Tags
#define XML_PEER_LIST_TAG "PEERS"
#define XML_PEER_TAG "PEER"
// Peering tags
#define XML_MONITOR_ADDR "MONITOR_IP_ADDR"
#define XML_MONITOR_PORT "MONITOR_PORT"
#define XML_MONITOR_VERSION "MONITOR_VERSION"
#define XML_MONITOR_AS "MONITOR_AS"
#define XML_MONITOR_ID "MONITOR_ID"
#define XML_ANNOUNCE_HOLDTIME "ANN_HOLDTIME"
#define XML_PEER_ADDR "PEER_IP_ADDR"
#define XML_PEER_PORT "PEER_PORT"
#define XML_PEER_VERSION "PEER_VERSION"
#define XML_PEER_AS "PEER_AS"
#define XML_PEER_ID "PEER_ID"
#define XML_PEER_MD5_PASS "MD5_PASSWD"
#define XML_MIN_HOLDTIME "MIN_HOLDTIME"
#define XML_PEER_ENABLED "ENABLED"
// Labeling tags
#define XML_LABEL_ACTION "LABEL_ACTION"
// Route Refresh tags
#define XML_RR_ACTION "ROUTE_REFRESH_ACTION"
// Capabilites
#define XML_ANNOUNCE_CAP_LIST "ANNOUNCE_CAPABILITY_LIST"
#define XML_RECEIVE_CAP_REQ_LIST "RECEIVE_CAPABILITY_REQ_LIST"
#define XML_CAPABILITY "CAPABILITY"
#define XML_CAPABILITY_CODE "CODE"
#define XML_CAPABILITY_ACTION "ACTION"
#define XML_CAPABILITY_LENGTH "LENGTH"
#define XML_CAPABILITY_VALUE "VALUE"
// Group
#define XML_GROUP_NAME_VALUE "GROUPNAME"

// Peer Groups Tags
#define XML_PEER_GROUP_LIST_TAG "PEER_GROUPS"
#define XML_PEER_GROUP_TAG "PEER_GROUP"
#define XML_PEER_GROUP_NAME_TAG "NAME"

// Periodic events tags
#define XML_PERIODIC_TAG "PERIODIC"
#define XML_STATUS_MSG_INTERVAL "PEER_STATUS_INTERVAL"
#define XML_ROUTE_REFRESH_INTERVAL "RIB_REFRESH_INTERVAL"
#define XML_SEND_ROUTE_REFRESH "SEND_ROUTE_REFRESH"

// XML Paths to various tags
#define XML_ROOT_PATH "//" XML_BGPMON_TAG "/"

// Login XML Path tags
#define XML_LOGIN_PATH XML_ROOT_PATH "/" XML_LOGIN_TAG
#define XML_LOGIN_PORT_PATH XML_LOGIN_PATH "/" XML_LOGIN_PORT_TAG
#define XML_LOGIN_ADDRESS_PATH XML_LOGIN_PATH "/" XML_LOGIN_ADDRESS_TAG
#define XML_LOGIN_ACCESS_PASSWORD_PATH XML_LOGIN_PATH "/" XML_LOGIN_ACCESS_PASSWORD_TAG
#define XML_LOGIN_ENABLE_PASSWORD_PATH XML_LOGIN_PATH "/" XML_LOGIN_ENABLE_PASSWORD_TAG
#define XML_LOGIN_CLI_ACL_PATH XML_LOGIN_PATH "/" XML_LOGIN_CLI_ACL_TAG
#define XML_LOGIN_CLIENT_UPDATE_ACL_PATH XML_LOGIN_PATH "/" XML_LOGIN_CLIENT_UPDATE_ACL_TAG
#define XML_LOGIN_CLIENT_RIB_ACL_PATH XML_LOGIN_PATH "/" XML_LOGIN_CLIENT_RIB_ACL_TAG
#define XML_LOGIN_MRT_ACL_PATH XML_LOGIN_PATH "/" XML_LOGIN_MRT_ACL_TAG

// Queue related XML Paths
#define XML_QUEUE_PATH XML_ROOT_PATH "/" XML_QUEUE_TAG
#define XML_QUEUE_PACING_ON_PATH XML_QUEUE_PATH "/" XML_QUEUE_PACING_ON
#define XML_QUEUE_PACING_OFF_PATH XML_QUEUE_PATH "/" XML_QUEUE_PACING_OFF
#define XML_QUEUE_ALPHA_PATH XML_QUEUE_PATH "/" XML_QUEUE_ALPHA
#define XML_QUEUE_MIN_WRITES_LIMIT_PATH XML_QUEUE_PATH "/" XML_QUEUE_MIN_WRITES_LIMIT
#define XML_QUEUE_PACING_INTERVAL_PATH XML_QUEUE_PATH "/" XML_QUEUE_PACING_INTERVAL
#define XML_QUEUE_LOG_INTERVAL_PATH XML_QUEUE_PATH "/" XML_QUEUE_LOG_INTERVAL

// Clients Control related XML Paths
#define XML_CLIENTS_CTR_PATH XML_ROOT_PATH "/" XML_CLIENTS_CTR_TAG
#define XML_CLIENTS_CTR_UPDATES_LISTEN_ADDR_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_UPDATES_LISTEN_ADDR
#define XML_CLIENTS_CTR_UPDATES_LISTEN_PORT_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_UPDATES_LISTEN_PORT
#define XML_CLIENTS_CTR_UPDATES_MAX_CLIENTS_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_UPDATES_MAX_CLIENTS
#define XML_CLIENTS_CTR_RIB_LISTEN_ADDR_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_RIB_LISTEN_ADDR
#define XML_CLIENTS_CTR_RIB_LISTEN_PORT_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_RIB_LISTEN_PORT
#define XML_CLIENTS_CTR_RIB_MAX_CLIENTS_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_RIB_MAX_CLIENTS
#define XML_CLIENTS_CTR_ENABLED_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_ENABLED
#define XML_CLIENTS_CTR_BGPMON_ID_PATH XML_CLIENTS_CTR_PATH "/" XML_CLIENTS_CTR_BGPMON_ID

// Mrts Control related XML Paths
#define XML_MRTS_CTR_PATH XML_ROOT_PATH "/" XML_MRTS_CTR_TAG
#define XML_MRTS_CTR_LISTEN_ADDR_PATH XML_MRTS_CTR_PATH "/" XML_MRTS_CTR_LISTEN_ADDR
#define XML_MRTS_CTR_LISTEN_PORT_PATH XML_MRTS_CTR_PATH "/" XML_MRTS_CTR_LISTEN_PORT
#define XML_MRTS_CTR_ENABLED_PATH XML_MRTS_CTR_PATH "/" XML_MRTS_CTR_ENABLED
#define XML_MRTS_CTR_MAX_MRTS_PATH XML_MRTS_CTR_PATH "/" XML_MRTS_CTR_MAX_MRTS
#define XML_MRTS_CTR_LABEL_ACTION_PATH XML_MRTS_CTR_PATH "/" XML_MRTS_CTR_LABEL_ACTION


// Chains related XML Paths
#define XML_CHAINS_PATH XML_ROOT_PATH "/" XML_CHAINS_LIST_TAG "/" XML_CHAIN_TAG

// Pees XML Paths
#define XML_PEERS_PATH XML_ROOT_PATH "/" XML_PEER_LIST_TAG "/" XML_PEER_TAG
// Pee Groups XML Paths
#define XML_PEER_GROUPS_PATH XML_ROOT_PATH "/" XML_PEER_GROUP_LIST_TAG "/" XML_PEER_GROUP_TAG

// Periodic events Paths
#define XML_PERIODIC_PATH XML_ROOT_PATH "/" XML_PERIODIC_TAG
#define XML_PERIODIC_STATUS_INTERVAL_PATH XML_PERIODIC_PATH "/" XML_STATUS_MSG_INTERVAL
#define XML_PERIODIC_RR_INTERVAL_PATH XML_PERIODIC_PATH "/" XML_ROUTE_REFRESH_INTERVAL
#define XML_PERIODIC_SEND_ROUTE_REFRESH_PATH XML_PERIODIC_PATH "/" XML_SEND_ROUTE_REFRESH 

#endif	// CONFIGDEFAULTS_H_
