/* 
 * 	Copyright (c) 2010 Colorado State University
 * 
 *	Permission is hereby granted, free of charge, to any person
 *	obtaining a copy of this software and associated documentation
 *	files (the "Software"), to deal in the Software without
 *	restriction, including without limitation the rights to use,
 *	copy, modify, merge, publish, distribute, sublicense, and/or
 *	sell copies of the Software, and to permit persons to whom
 *	the Software is furnished to do so, subject to the following
 *	conditions:
 *
 *	The above copyright notice and this permission notice shall be
 *	included in all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *	OTHER DEALINGS IN THE SOFTWARE.\
 * 
 * 
 *  File: buffer.h
 * 	Authors: Cathie Olschanowsky
 *
 *  Date: Dec. 2012
 */

#ifndef BUFFER_H_
#define BUFFER_H_

#include "log.h"
#include <stdlib.h>
#include <pthread.h>

#define START_BUFFER_SIZE 81920 // the default starting size (bytes) for 

struct buffer_struct{
  uint8_t  *buffer;        // a pointer to the backlog
  uint32_t size;           // starts out as 0 (in bytes)
  uint32_t start_size;      // starts out as 0 (in bytes)
  uint32_t start_pos; 
  uint32_t end_pos;
  uint8_t  *poss_end_msg;  
  uint8_t  poss_end_msg_start;
  uint8_t  poss_end_msg_end;
};
typedef struct buffer_struct buffer;

/******************************************************************************
* buffer_init
* Purpose: allocate the buffer and provide initial values
* Uses the default size defined by MRT_START_BACKLOG_SIZE
* Returns: 0 for success, 1 on failure
******************************************************************************/
int buffer_init(buffer *buf); 

/******************************************************************************
* buffer_init_size
* Purpose: allocate the buffer and provide initial values
* Takes a pointer to the backlog as well as the size in bytes 
* Returns: 0 for success, 1 on failure
******************************************************************************/
int buffer_init_size(buffer *backlog,uint32_t size); 

/******************************************************************************
* buffer_destroy
* Purpose: free allocated space, and destroy the lock
* Returns: 0 for success, 1 on failure
******************************************************************************/
int buffer_destroy(buffer *backlog); 

/******************************************************************************
* buffer_write
* Purpose: Write some number of bytes from a buffer to the backlog 
* Notes: This code assumes that the backlog lock has already been obtained!! 
*        If expansion of the buffer is needed, and not possible, messages
*        may be dropped.
* Returns: 0 for success, 1 on failure
******************************************************************************/
int buffer_write(buffer *backlog,uint8_t *buffer, uint32_t bytes);

/******************************************************************************
* buffer_read
* This function reads a single xml object from the backlog
* If the message that is next to be read will not fit in the space
* provided the lenght of that message is returned.
* Otherwise, 0 on success, -1 on failure
* The function can be called again with a larger buffer
******************************************************************************/
int buffer_read_XMLObj(buffer *backlog,uint8_t *rawMessage, uint16_t bytes);

#endif
