/*
** mrtReciever.c -- a stream socket server to receive MRT data and write it to file
**   based on the Beej's guide sample server
**   test the mrtPlayer by having it send MRT data to this receiver.  
**   mrtReciever will write the MRT data received to a file.   The resulting file should
**   exactly match the update files sent by the archiver
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>

// You can customize the following settings:
#define PORT "49999"  // the port mrtPlayer should connect to

#define FILENAME "receivedUpdates.bin"  // the file where mrtReceiver will write data

// Everything below here is part of the receive and should not be customized to a site
#define BACKLOG 10     // how many pending connections queue will hold
#define MAXDATASIZE 4096  // how big a buffer is used for receiving data 

void sigchld_handler(int s)
{
    while(waitpid(-1, NULL, WNOHANG) > 0);
}

// get sockaddr, IPv4 or IPv6:
void *get_in_addr(struct sockaddr *sa)
{
    if (sa->sa_family == AF_INET) {
        return &(((struct sockaddr_in*)sa)->sin_addr);
    }

    return &(((struct sockaddr_in6*)sa)->sin6_addr);
}

int main(void)
{
    int sockfd, new_fd;  // listen on sock_fd, new connection on new_fd
    struct addrinfo hints, *servinfo, *p;
    struct sockaddr_storage their_addr; // connector's address information
    socklen_t sin_size;
    struct sigaction sa;
    int yes=1;
    char s[INET6_ADDRSTRLEN];
    int rv;
    int numbytes;  
    char buf[MAXDATASIZE];

    memset(&hints, 0, sizeof hints);
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE; // use my IP

    if ((rv = getaddrinfo(NULL, PORT, &hints, &servinfo)) != 0) {
        fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(rv));
        return 1;
    }

    // loop through all the results and bind to the first we can
    for(p = servinfo; p != NULL; p = p->ai_next) {
        if ((sockfd = socket(p->ai_family, p->ai_socktype,
                p->ai_protocol)) == -1) {
            perror("mrtReceiver: socket");
            continue;
        }

        if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &yes,
                sizeof(int)) == -1) {
            perror("setsockopt");
            exit(1);
        }

        if (bind(sockfd, p->ai_addr, p->ai_addrlen) == -1) {
            close(sockfd);
            perror("mrtReceiver: bind");
            continue;
        }

        break;
    }

    if (p == NULL)  {
        fprintf(stderr, "mrtReceiver: failed to bind\n");
        return 2;
    }

    freeaddrinfo(servinfo); // all done with this structure

    if (listen(sockfd, BACKLOG) == -1) {
        perror("listen");
        exit(1);
    }

    sa.sa_handler = sigchld_handler; // reap all dead processes
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_RESTART;
    if (sigaction(SIGCHLD, &sa, NULL) == -1) {
        perror("sigaction");
        exit(1);
    }

    printf("mrtReceiver: waiting for connections...\n");

    while(1) {  // main accept() loop
        sin_size = sizeof their_addr;
        new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size);
        if (new_fd == -1) {
            perror("accept");
            continue;
        }

        inet_ntop(their_addr.ss_family,
            get_in_addr((struct sockaddr *)&their_addr),
            s, sizeof s);
        printf("mrtReciever: got connection from %s\n", s);

        if (!fork()) { // this is the child process
            close(sockfd); // child doesn't need the listener
            FILE *ptr_myfile;
            ptr_myfile=fopen(FILENAME,"wb");
            if (!ptr_myfile)
            {
                printf("Unable to open file FILENAME!");
		close(new_fd);
                exit(1);
             }
            numbytes = 1;
            while(numbytes > 0 ) {
                if ((numbytes = recv(new_fd, buf, MAXDATASIZE, 0)) == -1) {
	            perror("recv");
	            exit(1);
	        }
  	        printf("mrtReceiver: received %d bytes \n",numbytes);
		fwrite(buf, numbytes, 1, ptr_myfile);
            }
	    fclose(ptr_myfile);
            close(new_fd);
            exit(0);
        }
        close(new_fd);  // parent doesn't need this
    }

    return 0;
}
