/* 
 * 	Copyright (c) 2010 Colorado State University
 *	
 *	Permission is hereby granted, free of charge, to any person
 *	obtaining a copy of this software and associated documentation
 *	files (the "Software"), to deal in the Software without
 *	restriction, including without limitation the rights to use,
 *	copy, modify, merge, publish, distribute, sublicense, and/or
 *	sell copies of the Software, and to permit persons to whom
 *	the Software is furnished to do so, subject to the following
 *	conditions:
 *
 *	The above copyright notice and this permission notice shall be
 *	included in all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *	OTHER DEALINGS IN THE SOFTWARE.
 * 
 * 
 *  File: bgp_t.c
 *  Authors: Catherine Olschanowsky
 *  Date: Aug. 30, 2011
 */
#include <CUnit/Basic.h>
#include <stdlib.h>
#include <stdio.h>
#include "site_defaults.h"
#include "Util/bgp_t.h"
#include "Util/log.h"
#include "Labeling/rtable_t.h"
//#include "Mrt/mrtinstance_t.h"
#include "Mrt/mrtUtils_t.h"

/* The main() function for setting up and running the tests.
 * Returns a CUE_SUCCESS on successful running, another
 * CUnit error code on failure.
 */
int main()
{

  init_log ("test_driver", 0, 0, DEFAULT_LOG_FACILITY);
  CU_pSuite bgp_suite = NULL;
//  CU_pSuite mrtInstance_suite = NULL;
  CU_pSuite mrtUtil_suite = NULL; 
  CU_pSuite rtable_suite = NULL; 

 /* initialize the CUnit test registry */
 if (CUE_SUCCESS != CU_initialize_registry())
   return CU_get_error();

 /* add the suites to the registry */
 bgp_suite = CU_add_suite("BGP", init_bgp, clean_bgp);
 if (NULL == bgp_suite) {
    CU_cleanup_registry();
    return CU_get_error();
 }
 /*mrtInstance_suite = CU_add_suite("MRTinstance", init_mrtinstance, clean_mrtinstance);
 if (NULL == mrInstancet_suite) {
    CU_cleanup_registry();
    return CU_get_error();
 }*/
 mrtUtil_suite = CU_add_suite("MRTutil", init_mrtUtils, clean_mrtUtils);
 if (NULL == mrtUtil_suite) {
    CU_cleanup_registry();
    return CU_get_error();
 }
 rtable_suite = CU_add_suite("rtable", init_RTABLE, clean_RTABLE);
 if(NULL == rtable_suite){
   CU_cleanup_registry();
   return CU_get_error();
 }

   /* add the tests to the suites */
 if ((NULL == CU_add_test(bgp_suite, "test of BGP_createMessage",
                          testBGP_createMessage)) ||
      NULL == CU_add_test(bgp_suite, "test of BGP_addWithdrawnRouteToUpdate",
                          testBGP_addWithdrawnRouteToUpdate) ||
      NULL == CU_add_test(bgp_suite, "test of BGP_addNLRIToUpdate",
                          testBGP_addNLRIToUpdate) ||
      NULL == CU_add_test(bgp_suite, "test of BGP_addPathAttributeToUpdate",
                          testBGP_addPathAttributeToUpdate) ||
      NULL == CU_add_test(bgp_suite, "test of BGP_calculateLength",
                          testBGP_calculateLength) ||
      NULL == CU_add_test(bgp_suite, "test of BGP_serialize",testBGP_serialize)){

    CU_cleanup_registry();
    return CU_get_error();
 }
   /*if (( NULL == CU_add_test(mrtInstance_suite,"test MRT_createTableBufferFromType13Subtype1",testMRT_createTableBufferFromType13Subtype1)) ||
         NULL == CU_add_test(mrtInstance_suite,"test MRT_processType13SubtypeSpecific",testMRT_processType13SubtypeSpecific) ||
         NULL == CU_add_test(mrtInstance_suite,"test MRT_processType16SubtypeMessage",testMRT_processType16SubtypeMessage))
   {
      CU_cleanup_registry();
      return CU_get_error();
   }*/

   if (( NULL == CU_add_test(mrtUtil_suite,"test MRT_backlog_init",testMRT_backlog_init)) ||
        (NULL == CU_add_test(mrtUtil_suite,"test_MRT_backlog_write",testMRT_backlog_write)) ||
        (NULL == CU_add_test(mrtUtil_suite,"test_MRT_backlog_wrap",testMRT_backlog_wrap)) ||
        (NULL == CU_add_test(mrtUtil_suite,"test_MRT_backlog_fastforward",testMRT_backlog_fastforward)) ||
        (NULL == CU_add_test(mrtUtil_suite,"test_MRT_backlog_resize",testMRT_backlog_resize)) ||
        (NULL == CU_add_test(mrtUtil_suite,"test_MRT_backlog_read",testMRT_backlog_read)))
   {
      CU_cleanup_registry();
      return CU_get_error();
   }
 // rtable tests
 if ((NULL == CU_add_test(rtable_suite, "test of rtable_printPrefixV4",
                          testRTABLE_printPrefixV4)) ||
     (NULL == CU_add_test(rtable_suite, "test of rtable_printPrefixV6",
                          testRTABLE_printPrefixV6)) ||
     (NULL == CU_add_test(rtable_suite, "test of rtable_stringToPrefixV4",
                          testRTABLE_stringToPrefixV4)) ||
     (NULL == CU_add_test(rtable_suite, "test of rtable_stringToPrefixV6",
                          testRTABLE_stringToPrefixV6))){
    CU_cleanup_registry();
    return CU_get_error();
 }

 /* Run all tests using the CUnit Basic interface */
 CU_basic_set_mode(CU_BRM_VERBOSE);
 CU_basic_run_tests();
 CU_cleanup_registry();
 return CU_get_error();
}

